#include <QtGui/QtGui>
#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QFrame>
#include <QtGui/QGridLayout>
#include <QtGui/QHBoxLayout>
#include <QtGui/QLabel>
#include <QtGui/QListView>
#include <QtGui/QListWidgetItem>
#include <QtGui/QProgressBar>
#include <QtGui/QPushButton>
#include <QtGui/QTextBrowser>
#include <QtGui/QTreeWidget>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>

#include "test_register_function.h"
#include "test_functions.h"
#include "test_gui.h"

namespace Tester
{
  TestMainWindow::TestMainWindow(const QString title)
  {
    setupUi(this);
    createActions();

    fillTree();

    setWindowTitle(title);
  }

  void TestMainWindow::setupUi(QMainWindow *MainWindow)
  {
    MainWindow->resize(QSize(414, 650).expandedTo(MainWindow->minimumSizeHint()));
    centralWidget = new QWidget(MainWindow);
    centralWidget->setObjectName(QString::fromUtf8("centralWidget"));
    gridLayout = new QGridLayout(centralWidget);
    gridLayout->setSpacing(6);
    gridLayout->setMargin(8);
    gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
    displayInfoFrame = new QFrame(centralWidget);
    displayInfoFrame->setObjectName(QString::fromUtf8("displayInfoFrame"));
    displayInfoFrame->setFrameShape(QFrame::StyledPanel);
    displayInfoFrame->setFrameShadow(QFrame::Raised);
    verticalDisplayLayout = new QVBoxLayout(displayInfoFrame);
    verticalDisplayLayout->setSpacing(6);
    verticalDisplayLayout->setMargin(8);
    verticalDisplayLayout->setObjectName(QString::fromUtf8("verticalDisplayLayout"));
    horizontalDisplayLayout = new QHBoxLayout();
    horizontalDisplayLayout->setSpacing(6);
    horizontalDisplayLayout->setMargin(0);
    horizontalDisplayLayout->setObjectName(QString::fromUtf8("horizontalDisplayLayout"));
    verticalButtonDisplayLayout = new QVBoxLayout();
    verticalButtonDisplayLayout->setSpacing(6);
    verticalButtonDisplayLayout->setMargin(0);
    listView = new QListWidget(displayInfoFrame);
    listView->setObjectName(QString::fromUtf8("listView"));
  
    horizontalDisplayLayout->addWidget(listView);
  
    runButton = new QPushButton(displayInfoFrame);
    runButton->setObjectName(QString::fromUtf8("runButton"));

    verticalButtonDisplayLayout->addWidget(runButton);
    horizontalDisplayLayout->addLayout(verticalButtonDisplayLayout);
  
  
    verticalDisplayLayout->addLayout(horizontalDisplayLayout);
  
    textBrowser = new QTextEdit(displayInfoFrame);
    textBrowser->setObjectName(QString::fromUtf8("textBrowser"));
  
    verticalDisplayLayout->addWidget(textBrowser);
  
  
    gridLayout->addWidget(displayInfoFrame, 3, 0, 1, 1);
  
    frame = new QFrame(centralWidget);
    frame->setObjectName(QString::fromUtf8("frame"));
    frame->setFrameShape(QFrame::StyledPanel);
    frame->setFrameShadow(QFrame::Raised);
    gridLayout1 = new QGridLayout(frame);
    gridLayout1->setSpacing(6);
    gridLayout1->setMargin(8);
    gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
    textLayout = new QGridLayout();
    textLayout->setSpacing(6);
    textLayout->setMargin(0);
    textLayout->setObjectName(QString::fromUtf8("textLayout"));
    nb_errors = new QLabel(frame);
    nb_errors->setObjectName(QString::fromUtf8("nb_errors"));
  
    textLayout->addWidget(nb_errors, 1, 1, 1, 1);
  
    labelFailures = new QLabel(frame);
    labelFailures->setObjectName(QString::fromUtf8("labelFailures"));
  
    textLayout->addWidget(labelFailures, 0, 2, 1, 1);
  
    nb_tests = new QLabel(frame);
    nb_tests->setObjectName(QString::fromUtf8("nb_tests"));
  
    textLayout->addWidget(nb_tests, 1, 0, 1, 1);
  
    labelTests = new QLabel(frame);
    labelTests->setObjectName(QString::fromUtf8("labelTests"));
  
    textLayout->addWidget(labelTests, 0, 0, 1, 1);
  
    labelErrors = new QLabel(frame);
    labelErrors->setObjectName(QString::fromUtf8("labelErrors"));
  
    textLayout->addWidget(labelErrors, 0, 1, 1, 1);
  
    nb_failures = new QLabel(frame);
    nb_failures->setObjectName(QString::fromUtf8("nb_failures"));
  
    textLayout->addWidget(nb_failures, 1, 2, 1, 1);
  
  
    gridLayout1->addLayout(textLayout, 1, 0, 1, 1);
  
    progressBar = new QProgressBar(frame);
    progressBar->setObjectName(QString::fromUtf8("progressBar"));
    progressBar->setValue(24);
  
    gridLayout1->addWidget(progressBar, 0, 0, 1, 1);
  
  
    gridLayout->addWidget(frame, 2, 0, 1, 1);
    tree = new QTreeWidget(centralWidget);
    tree->setObjectName(QString::fromUtf8("tree"));
    tree->setColumnCount(1);
    tree->header()->hide();

    gridLayout->addWidget(tree, 0, 0, 2, 1);
  
    MainWindow->setCentralWidget(centralWidget);
    retranslateUi(MainWindow);
  
    iconFailure = new QIcon(QString::fromUtf8("images/failure.png"));
    iconError = new QIcon(QString::fromUtf8("images/error.png"));
  
      //QMetaObject::connectSlotsByName(MainWindow);
  } // setupUi

  void TestMainWindow::retranslateUi(QMainWindow *MainWindow)
  {
    runButton->setText(QApplication::translate("MainWindow", "Run"));
    labelFailures->setText(QApplication::translate("MainWindow", "Failures :"));
    labelTests->setText(QApplication::translate("MainWindow", "Tests :"));
    labelErrors->setText(QApplication::translate("MainWindow", "Errors :"));
  
    clear();
    Q_UNUSED(MainWindow);
  } // retranslateUi

  void TestMainWindow::clear()
  {
    tests = 0;
    errors = 0;
    failures = 0;
    nb_tests->setText(QApplication::translate("MainWindow", "0/0"));
    nb_errors->setText(QApplication::translate("MainWindow", "0"));
    nb_failures->setText(QApplication::translate("MainWindow", "0"));
    progressBar->setRange(0, 0);
    progressBar->setValue(0);
    listView->clear();
    errorsFailuresReport.clear();
    textBrowser->clear();
    textBrowser->setReadOnly(true);
  }

  void TestMainWindow::createActions()
  {
    connect(runButton, SIGNAL(clicked()), this, SLOT(runTest()));
    connect(tree, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(treeTestChanged(QTreeWidgetItem*, QTreeWidgetItem*)));
    connect(listView, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(testChanged(QListWidgetItem*)));
  }

  void TestMainWindow::runTest()
  {
    clear();

    std::list<TestFunctionStruct*> testsList = getSelectedTestStruct((tree->currentItem() ? tree->currentItem() : tree->headerItem()))->getRealTests();

    size_tests = testsList.size();
    progressBar->setRange(0, size_tests);
    progressBar->setValue(0);

    for(std::list<TestFunctionStruct*>::iterator i = testsList.begin(); i != testsList.end(); ++i)
    {
      ++tests;
      try
      {
        (**i)();
      }
      catch(Thrown& Failure)
      {
        addFailure(Failure);
      }
      catch(const std::exception& exception)
      {
        addError((**i).functionName, QString::fromAscii(exception.what()));
      }
      catch(...)
      {
        addError((**i).functionName);
      }

      updateProgressBar();
    }
  }

  void TestMainWindow::testChanged(QListWidgetItem* item)
  {
    if(item != NULL)
    {
      textBrowser->clear();
      textBrowser->append(errorsFailuresReport[listView->currentRow()]);
    }
  }

  void TestMainWindow::treeTestChanged(QTreeWidgetItem* current, QTreeWidgetItem* previous)
  {
    size_tests = getSelectedTestStruct(current)->nbTests();
    nb_tests->setText(QString::number(0U) + QString::fromUtf8("/") + QString::number(size_tests));
    nb_errors->setText(QString::number(0U));
    nb_failures->setText(QString::number(0U));
  }

  void TestMainWindow::fillTree()
  {
    TestManager& instance = TestManager::getInstance();
    QTreeWidgetItem* root = new QTreeWidgetItem();
    root->setText(0, "All Tests");
    tree->addTopLevelItem(root);

    for(std::map<QString, TestFunctionStruct*>::iterator subNode = instance.getRoot()->getLeaves().begin(); subNode != instance.getRoot()->getLeaves().end(); ++subNode)
    {
      addToTree(root, *subNode);
    }
    tree->setItemExpanded(root, true);
  }

  void TestMainWindow::addToTree(QTreeWidgetItem* node, std::pair<const QString, TestFunctionStruct*>& subTree)
  {
    QTreeWidgetItem* subNode = new QTreeWidgetItem(node);
    subNode->setText(0, subTree.first);
    if(subTree.second->hasLeaves())
    {
      for(std::map<QString, TestFunctionStruct*>::iterator leaf = subTree.second->getLeaves().begin(); leaf != subTree.second->getLeaves().end(); ++leaf)
      {
        addToTree(subNode, *leaf);
      }
    }
  }

  void TestMainWindow::addFailure(const Thrown& failure)
  {
    failures++;
    nb_failures->setText(QString::number(failures));
    QListWidgetItem* listItem = new QListWidgetItem(failure.function + "\n" + failure.title + "\n", listView);
    listItem->setIcon(*iconFailure);
    errorsFailuresReport.push_back(failure.function + "\n" + failure.explanation);
  }

  void TestMainWindow::addError(const QString& function, const QString comment)
  {
    errors++;
    nb_errors->setText(QString::number(errors));
    QListWidgetItem* listItem = new QListWidgetItem(QString::fromUtf8("Unexpected error"), listView);
    listItem->setIcon(*iconError);
    errorsFailuresReport.push_back(function + "\n" + comment);
  }

  void TestMainWindow::addError(const QString& function)
  {
    errors++;
    nb_errors->setText(QString::number(errors));
    QListWidgetItem* listItem = new QListWidgetItem(QString::fromUtf8("Unexpected error"), listView);
    listItem->setIcon(*iconError);
    errorsFailuresReport.push_back(function);
  }

  void TestMainWindow::updateProgressBar()
  {
    progressBar->setValue(progressBar->value() + 1);
    nb_tests->setText(QString::number(tests) + QString::fromUtf8("/") + QString::number(size_tests));
    update();
  }

  QString TestMainWindow::path(QTreeWidgetItem* element)
  {
    if(element->parent() == NULL)
    {
      return "All Tests";
    }
    else
    {
      return path(element->parent()) + ":" + element->text(0);
    }
  }

  TestFunctionStruct* TestMainWindow::getSelectedTestStruct(QTreeWidgetItem* element)
  {
    QStringList pathToTake = path(element).split(':');

    pathToTake.pop_front();
    return TestManager::getInstance().getTestFunctionStructFromQStringList(pathToTake);
  }
}
